/*
 * various configuration and utilities routines 
 */

#include  <stdio.h>
#include  <stdlib.h>
#include  <string.h>
#include  <time.h>
#include  <linux/joystick.h>

#include  "contexte.h"
#include  "jmidi.h"

/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + joymidi + + */
/* hop, we are in a hurry, so we can safely use global variables */

static TransJ2M		translations[NB_MAX_TRANS];
static ButtonName	buttons_names[NB_MAX_TRANS];
static int		nbre_named_buttons;
static int		nbre_translations;

/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + joymidi + + */
static int load_the_keys(FILE *fpfrom)
{
char		chaine[T_LINE_CTX+1], *cptr, *cptr2;
long		key_num, value;

#if DEBUG_LEVEL
fprintf(stderr, "%s (( %p ))\n", __func__, fpfrom);
#endif

while (NULL!=fgets(chaine, T_LINE_CTX, fpfrom))
	{
	if (strlen(chaine) && chaine[strlen(chaine)-1]=='\n')
		chaine[strlen(chaine)-1]='\0';
#if DEBUG_LEVEL 
	fprintf(stderr, "%s : line is [%s]\n", __func__, chaine);
#endif

	cptr = strtok(chaine, " \t");
	if ((NULL==cptr)||('#'==cptr[0])) /*  empty line or comment */
		continue;

	if ('.' == cptr[0])
		{
		/* end of section */
		fprintf(stderr, "end keys section : %d found\n",
						key_num);
		break;
		}

	/* OK, filtered all the spurious case, trying to get a valid
	 * button name|numbre */
	key_num = strtol(cptr, &cptr2, 0);
	if (cptr==cptr2)
		{
		/* not a valid number... seeking for a name */
		key_num = lookup_button_number(cptr);	
		}
#if DEBUG_LEVEL
	fprintf(stderr, "value is %ld\n", key_num);
#endif
	if ((key_num < 0) || (key_num > NB_MAX_TRANS))
		{
		fprintf(stderr, "button number %ld is invalid\n", key_num);
		exit(1);
		}

	/* got a usable button index, reading the associated note */
	cptr = strtok(NULL, " \t");
	value = strtol(cptr, &cptr2, 0);
	if (cptr==cptr2)
		{
		/* not a valid number... doing an ABEND */
		fprintf(stderr, "sorry on [%s]\n", cptr);
		abort();
		}
	translations[key_num].note = value;

	/* FIXME now, we look for any additional informations,
 	   eg : an alternate midi channel or a specific velocity */


	if (key_num >= NB_MAX_BUTTONS)
		{
		fprintf(stderr, "too much keys, sorry\n");
		abort();
		}

	key_num += 1;		/* going to the next */
	}

return key_num-1;
}
/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + joymidi + + */
int load_translations(char *fname)
{
char		chaine[T_LINE_CTX+1], *cptr, *cptr2;
FILE		*fp;
int		bouton, note, foo;

#if DEBUG_LEVEL
fprintf(stderr, "%s (( %s ))\n", __func__, fname);
#endif
if (NULL==(cptr=where_is_dotrc()) )
	{
	/* goodbye, cruel world */
	fprintf(stderr, "%s : fatal error, aborting\n", __func__);
	abort();
	}
#if DEBUG_LEVEL
fprintf(stderr, "%s : context in '%s'\n", __func__, cptr);
#endif
strcpy(chaine, cptr);

/* may be check if we have some valid space in 'path' ? */

strcat(chaine, fname);
#if DEBUG_LEVEL
fprintf(stderr, "%s : path is '%s'\n", __func__, chaine);
#endif

if (NULL==(fp=fopen(chaine, "r")))
	{
	fprintf(stderr, "%s : can not open %s\n", __func__, chaine);
	return 0;
	}

/* initialising the array with some defaults values */
for (foo=0; foo<NB_MAX_TRANS; foo++)
	{
	translations[foo].channel = -1;		/* use default */
	translations[foo].note = -1;		/* invalid content */
	}
nbre_translations = 0;


while (NULL!=fgets(chaine, T_LINE_CTX, fp))
	{
	if (strlen(chaine) && chaine[strlen(chaine)-1]=='\n')
		chaine[strlen(chaine)-1]='\0';
#if DEBUG_LEVEL 
	fprintf(stderr, "line is [%s]\n", chaine);
#endif
	cptr = strtok(chaine, " \t");
	if ((NULL==cptr)||('#'==cptr[0])) /*  empty line or comment */
		continue;
	cptr2 = strtok(NULL, " \t");
#if DEBUG_LEVEL
	fprintf(stderr, "[%s]  [%s]\n", cptr, cptr2);
#endif

	if	(!strcmp("asciiupload", cptr))
		{
		fprintf(stderr, "'asciiupload' is not yet implemented.\n");
		}
	else if	(!strcmp("binupload", cptr))
		{
		fprintf(stderr, "'binupload' is not yet implemented.\n");
		}
	else if (!strcmp("keys", cptr))
		{
		fprintf(stderr, "====== loading the keys =====\n");
		foo = load_the_keys(fp);
		fprintf(stderr, "load_the_keys -> %d\n", foo);
		fprintf(stderr, "got %d translations\n", nbre_translations);
		}
	else if (!strcmp("runtest", cptr))
		{
		fprintf(stderr, "running a test is not yet implemented.\n");
		}
	else	/* failure */
		{
		fprintf(stderr, "'%s' is unknow here, sorry...\n", cptr);
		exit(1);
		}
	}

fclose(fp);

/* may be we can report an error status.count ? */

return 0;
}
/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + joymidi + + */
int get_translation(int button)
{
int	note;

#if DEBUG_LEVEL
fprintf(stderr, "%s (( %d ))\n", __func__, button);
#endif

if ( (button >= 0) && (button<NB_MAX_TRANS) )
	{
	note = translations[button].note;
	}
else
	{
	fprintf(stderr, "button #%d out of range\n", button);
	note = -1;	/* FAIL */
	}

return note;
}
/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + joymidi + + */
int dump_translations(char *txt, int numeric)
{
int	foo;

if ( numeric )
	{
	fprintf(stderr, "%s : bad call\n", __func__);
	abort();
	}

printf("+------ dump of translations ----+\n");
for (foo=0; foo<NB_MAX_TRANS; foo++)
	{
	if ( translations[foo].note < 0 )
		{
#if DEBUG_LEVEL > 1
		fprintf(stderr, "translation %d not confed\n", foo);
#endif
		continue;
		}

	printf("| %4d  |   %3d\n", foo, translations[foo].note);

	}
printf("+--------------------------------+\n");

return 0;
}
/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + joymidi + + */
char *jsevent2text(int event)
{
static char chaine[30];

switch (event)
	{
	case JS_EVENT_BUTTON | JS_EVENT_INIT:	return "button init";
	case JS_EVENT_AXIS | JS_EVENT_INIT:	return "axis init";
	case JS_EVENT_BUTTON:			return "button";
	case JS_EVENT_AXIS:			return "axis";
	default: 
		sprintf(chaine, "0x%02X is a fail", event);
		return chaine;
	}
return NULL;
}
/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + joymidi + + */
int load_buttons_names(char *fname)
{
char		chaine[T_LINE_CTX+1], *cptr, *cptr2, *name;
FILE		*fp;
int		value, current, event_type;

#if DEBUG_LEVEL
fprintf(stderr, ">>>> %s (( %s ))\n", __func__, fname);
#endif

if (NULL==(cptr=where_is_dotrc()) )
	{
	/* goodbye, cruel world */
	fprintf(stderr, "%s : fatal error, aborting\n", __func__);
	abort();
	}
#if DEBUG_LEVEL
fprintf(stderr, "%s : context in '%s'\n", __func__, cptr);
#endif
strcpy(chaine, cptr);

/* may be check if we have some space in 'path' ? */

strcat(chaine, fname);
#if DEBUG_LEVEL
fprintf(stderr, "%s : fullpath '%s'\n", __func__, chaine);
#endif

if (NULL==(fp=fopen(chaine, "r")))
	{
	fprintf(stderr, "%s : can not open %s\n", __func__, chaine);
	return 0;
	}

/* OK the file is opened, so we can try to read it... */

current = 0;		/* start from the beginning */

while (NULL!=fgets(chaine, T_LINE_CTX, fp))
	{
	if (strlen(chaine) && chaine[strlen(chaine)-1]=='\n')
		chaine[strlen(chaine)-1]='\0';
	else
		{
		fprintf(stderr, "%s : line too long\n", fname);
		exit(1);
		}
#if DEBUG_LEVEL > 1
	fprintf(stderr, "line is [%s]\n", chaine);
#endif

	cptr = strtok(chaine, " \t");
	if (NULL == cptr)			/* empty line */
		continue;
	if ('#' == cptr[0])			/* comment line */
		continue;

	/* read the keywords : button/axis/whatelse */
	if ( ! strcmp("button", cptr) )
		{
		event_type = JS_EVENT_BUTTON;
		}
	else if ( ! strcmp("axis", cptr) )
		{
		event_type = JS_EVENT_AXIS;
		}
	else
		{
		fprintf(stderr, "%s : borking on '%s'\n", __func__, cptr);
		exit(1);
		}
	
#if DEBUG_LEVEL
	fprintf(stderr, "%10s ->  %d\n", cptr, event_type);
#endif
	buttons_names[current].type = event_type;

	cptr = strtok(NULL, " \t");
	name = cptr;
#if DEBUG_LEVEL > 1
	printf("name is [%s]\n", name);
#endif

	cptr = strtok(NULL, " \t");
	value = strtol(cptr, &cptr2, 0);
#if DEBUG_LEVEL > 1
	printf("number is [%s] value is %d\n", cptr, value);
#endif

	/* Ok whe have get all the information */
	strcpy(buttons_names[current].name, name);
	buttons_names[current].num = value;
	current++;
	}

nbre_named_buttons = current;

#if DEBUG_LEVEL
fprintf(stderr, "%s : got %d buttons names\n", __func__, current);
#endif

fclose(fp);

return 1;
}
/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + joymidi + + */
int dump_buttons_names(void)
{
int	foo;
char	type;

#if DEBUG_LEVEL
fprintf(stderr, ">>>> %s (( void ))\n", __func__);
fprintf(stderr, "     we have %d buttons\n", nbre_named_buttons);
#endif

for (foo=0; foo<nbre_named_buttons; foo++)
	{
	switch (buttons_names[foo].type)
		{
		case JS_EVENT_BUTTON:	type = 'B';	break;
		case JS_EVENT_AXIS:	type = 'A';	break;
		default:		type = '?';	break;
		}
	printf("%4d    |   '%c'  %3d   %s\n", foo,
				type,
				buttons_names[foo].num,
				buttons_names[foo].name);
	}

return 0;
}
/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + joymidi + + */
int  lookup_button_number(char *name)
{
int	foo;
long	val;

#if DEBUG_LEVEL 
fprintf(stderr, ">>>> %s (( '%s' ))  ", __func__, name);
fprintf(stderr, "  have %d buttons\n", nbre_named_buttons);
#endif

val = -1;
for (foo=0; foo<nbre_named_buttons; foo++)
	{
#if DEBUG_LEVEL > 1
	fprintf(stderr, "     %3d ?= '%s'\n", foo, buttons_names[foo].name);
#endif

	if (!strcmp(name, buttons_names[foo].name))
		{
		val = buttons_names[foo].num;
		break;
		}
	}
return val;
}
/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + joymidi + + */
char *lookup_button_name(int num, int type)
{
int	foo;

#if DEBUG_LEVEL > 1
fprintf(stderr, ">>>> %s (( %d 0x%x ))\n", __func__, num, type);
#endif

type &= ~JS_EVENT_INIT;
#if DEBUG_LEVEL > 1
fprintf(stderr, "    type = 0x%02x\n", type);
#endif

for (foo=0; foo<nbre_named_buttons; foo++)
	{
	if ( (buttons_names[foo].num == num) &&
             (buttons_names[foo].type == type) )
		{
		return buttons_names[foo].name;
		}
	}
return NULL;
}
/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + joymidi + + */
int millisleep(int ms)
{
struct timespec ts;
int		foo;

#if DEBUG_LEVEL
fprintf(stderr, ">>>> %s (( %d ))\n", __func__, ms);
#endif

ts.tv_sec = ms / 1000;
ts.tv_nsec = (ms % 1000) * 1000 * 1000;

#if DEBUG_LEVEL
fprintf(stderr, "     %9ld   %9ld\n", (long)ts.tv_sec, ts.tv_nsec);
#endif

foo = nanosleep(&ts, NULL);
if (foo)
	{
	perror("millisleep fail !");
	exit(1);
	}

return 0;
}
/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + joymidi + + */
