#include  <stdio.h>
#include  <stdlib.h>
#include  <unistd.h>
#include  <string.h>
#include  <sys/types.h>
#include  <sys/stat.h>
#include  <fcntl.h>
#include  <linux/joystick.h>

#include  "contexte.h"
#include  "jmidi.h"
#include  "alsa_out.h"

char	*default_joydevice = "/dev/input/js0";
char	joydevice[T_LINE_CTX+1];
int	verbose;

/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + joymidi + + */
int please_load_the_setup(char *txt)
{
int			foo;
char			texte[T_LINE_CTX+1];

#if DEBUG_LEVEL
fprintf(stderr, ">>>> %s (( '%s' ))\n", __func__, txt);
#endif

foo = init_context(txt);
#if DEBUG_LEVEL
fprintf(stderr, "     init_contexte -> %d\n", foo);
#endif

if ( ! get_context("buttonsnames", texte) )
	{
	fprintf(stderr, "bad context file\n");
	exit(1);
	}
foo = load_buttons_names(texte);
#if DEBUG_LEVEL
fprintf(stderr, "     load buttons names -> %d\n", foo);
dump_buttons_names();
#endif


if ( ! get_context("translation", texte) )
	{
	fprintf(stderr, "bad context file, no translation ?\n");
	exit(1);
	}
printf("OK, the translation file is '%s'\n", texte);
foo = load_translations(texte);
printf("chargement de la translation -> %d\n", foo);

#if DEBUG_LEVEL > 1
foo = dump_translations("yo man, tout baigne ?", 0);
fprintf(stderr, "     dump de la translation -> %d\n", foo);
#endif

return 0;	/* what's te meaning of this value ? XXX */
}
/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + joymidi + + */
/* TODO here we have to select the right output method :
 *                                - rawmidi
 *                                - alsaseq
 */
int send_a_note(int button, int onoff)
{
int	note;

#if DEBUG_LEVEL
fprintf(stderr, ">>>> %s (( btn:%d %-3s ))\n", __func__, button,
				onoff ? "on" : "off");
#endif

if (button < 0)
	{
	fprintf(stderr, "fatal error : number is %d\n", button);
	abort();
	}

note = get_translation(button);
if (note <= 0)
	{
	fprintf(stderr, "in %s, button %d is not translated\n",
			__func__,	button);
	return -1;
	}

if (onoff)
	send_alsa_note_on(note, 100);		
else
	send_alsa_note_off(note);		

return -1;
}
/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + joymidi + + */
int send_a_control(int button, int val)
{
int	note;

#if DEBUG_LEVEL
fprintf(stderr, ">>>> %s (( btn:%d val:%-6d ))\n", __func__, button, val);
#endif

return 0;
}
/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + joymidi + + */
/*
 * yo, here we are...
 */
int this_is_the_main_loop(int this_is_joy)
{
struct js_event		evn;
int			foo;
char			*cptr;

foo = open_alsa_out("joymidi", 1);

/*
 * and now, we enter the big eternal loop...
 */
for (;;)
	{
	/* hint: may be use a select, so we can sent some sort of
 	 * keep-alive or alert signal to the device ? */

	foo = read(this_is_joy, &evn, sizeof(evn));
	if (verbose)
		{
		cptr = lookup_button_name(evn.number, evn.type);
		if (NULL==cptr)	cptr = "gni?";
		printf("%12u:   %-12s   %-16s      %6d\n",
				evn.time, jsevent2text(evn.type),
				cptr, evn.value);
		}

	switch(evn.type)
		{
		case JS_EVENT_AXIS:
			send_a_control(evn.number, evn.value);
			break;

		case JS_EVENT_BUTTON:
			send_a_note(evn.number, evn.value);
			break;

		default:
			fprintf(stderr, "joystick event 0x%02x ignored\n",
							 evn.type);
			break;
		}

	}

return 0;
}
/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + joymidi + + */
void help(int k)
{
fprintf(stderr, 
	"joymidi options:\n"
	"  -v : increment verbose level\n"
	"  -h : get more help (if available)\n"
	"  -i : set the locations of init files\n"
	"  -d : use an alternate joy device\n"
	"  -n : display just the numbers\n");
exit(1);
}
/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + joymidi + + */
int main(int argc, char *argv[])
{
int			fd_joy, foo;
char			options[] = "vhi:d:";
char			joydevice[T_LINE_CTX+1];
char			initlocation[T_LINE_CTX+1], rcdir;

printf("\n******************* JOYMIDI *********************\n");

joydevice[0] = '\0';
initlocation[0] = '\0';

/* exploration of the command line arguments */
while ((foo = getopt(argc, argv, options)) != -1)
	{
	switch(foo) {
	case 'v':
		verbose++;
		break;
	case 'd':
		strncpy(joydevice, optarg, T_LINE_CTX);
		break;
	case 'h':
		help(1);
		break;
	case 'i':
		strncpy(initlocation, optarg, T_LINE_CTX);
		fprintf(stderr, "* init files in '%s'\n", initlocation);
		break;
	default:
		/* XXX help(0); */
		break;
		}
	}

please_load_the_setup(initlocation);

if ( ! get_context("joydevice", joydevice) )
	{
	fprintf(stderr, "nasty context file\n");
	exit(1);
	}
#if DEBUG_LEVEL
fprintf(stderr, "trying to open '%s'\n", joydevice);
#endif

fd_joy = open(joydevice, O_RDONLY);
if (fd_joy < 0)
	{
	perror(joydevice);
	exit(1);
	}
#if DEBUG_LEVEL
fprintf(stderr, "ouverture manette '%s' --> %d\n", joydevice, fd_joy);
#endif

if (verbose)
	{
	printf("joymidi (pid:%d) run in verbose mode\n", getpid());
	}

foo = this_is_the_main_loop(fd_joy);

close(fd_joy);

return 0;
}

/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + joymidi + + */

